﻿/**
* ...
* @author Mikhail Ivanchuk (http://dcafe.ru)
* @version 0.1
* @param
*/

class com.stuffedguys.tracker.Tracker {
	public var log:Boolean = false;
	private var trackerSiteHost:String;
	private var trackerDefaultURL:String;
	private var trackerActionURL:String;
	private var trackerSaleURL:String;
	private var siteID:Number;
	private var siteTextID:String;

	private var _listeners:Array;

	private var parameters:Object;
	private var backup_parameters:Object;
	private var _charset:String;
	private var flash_parameters:Object;


	//private var currentPage:Object;

/**
* CONSTRUCTOR
* @param	lTrackerSiteHost	: host of a site
* @param	lTrackerDefaultURL	: URL for reporting track()
* @param	lTrackerActionURL	: URL for reporting trackAction()
* @param	lTrackerSaleURL		: URL for reporting trackSale()
* @param	lSiteID				: ID of a site
* @param	lSiteTextId			: text ID of a site
*/
	public function Tracker	(
							lTrackerSiteHost:String,
							lTrackerDefaultURL:String,
							lTrackerActionURL:String,
							lTrackerSaleURL:String,
							lSiteID:Number,
							lSiteTextID:String
							)
	{
		trackerSiteHost = checkForHTTP(lTrackerSiteHost);
		trackerDefaultURL = checkForHTTP(lTrackerDefaultURL);
		trackerActionURL = checkForHTTP(lTrackerActionURL);
		trackerSaleURL = checkForHTTP(lTrackerSaleURL);
		siteID = lSiteID;
		if (lSiteTextID !="" && lSiteTextID != undefined) siteTextID = lSiteTextID;
		// parameters are reported when changed. (parameters !=backup_parameters)
		parameters = {vname: null, vtag: null, overwriteTags: null, presetIP: null, presetID: null};

		backup_parameters = {vname: null, vtag: null, overwriteTags: null, presetIP: null, presetID: null};

		_charset = "utf-8";
		// flash parameters are reported each time tracer do report
		// wr: resolution (cheak every time tracker do report)
		// fl: version of a flash player
		// ft: code for tracker to indicate flash tracker.
		flash_parameters = {wr: null, fl: System.capabilities.version, ft: 1};

		_listeners = [];
	}
/**
* TRACK NAVIGATION
* @param	currentPage: A String describing the path of a user on a site. Can be an Object {Addr: [path], Title: [title of a page]}
*/
	public function track(lCurrentPage:Object):Void {
		// standartizing currentPage Object.
		var tracker_reference:Tracker = this;
		var currentPage:Object = parseCurrentPageProperties(lCurrentPage);

		if (currentPage.Addr == undefined || currentPage.Addr == "") {
			broadcastError("ERROR [track()]. CurrentPage Object was not set properly.");
			return;
		}
		// building report variables
		var track_lv:LoadVars = new LoadVars();
		track_lv.cur = standartize(currentPage.Addr);
		if (currentPage.Title != null) track_lv.title = currentPage.Title;
		// additional parameters if needed
		addParameters(track_lv);
		track_lv.sendAndLoad(trackerDefaultURL,track_lv,"POST");
		track_lv.onLoad = function(success:Boolean) {
			tracker_reference.broadcastReport(success,"track()",currentPage.Addr);
			delete this;
			}
		if (log) {
			broadcastLog("track()",trackerDefaultURL,track_lv);
		}

	}

/**
* TRACK ACTION
* @param	currentPage: A String describing the path of a user on a site. Can be an Object {path: [path], title: [title of a page]}
* @param	actionID
* @param	actionItem
*/
	public function trackAction	(
								lCurrentPage:Object,
								actionID:Number,
								actionItem:String
								):Void {


		var tracker_reference:Tracker = this;
		var currentPage:Object = parseCurrentPageProperties(lCurrentPage);

		if (currentPage.Addr == undefined || currentPage.Addr == "") {
			broadcastError("ERROR [trackAction()]. CurrentPage Object was not set properly.");
			return;
		}
		if (actionID == undefined || isNaN(actionID)) {
			broadcastError("ERROR [trackAction()]. actionID was not set properly.");
			return;
		}

		var track_action_lv:LoadVars = new LoadVars();
		track_action_lv.cur = standartize(currentPage.Addr);
		if (currentPage.Title != null) track_action_lv.title = currentPage.Title;

		track_action_lv.eid = actionID.toString();
		if (actionItem != undefined && actionItem!="") track_action_lv.itm = actionItem;

		track_action_lv.code = 1;
		// additional parameters if needed
		addParameters(track_action_lv);
		track_action_lv.sendAndLoad(trackerActionURL,track_action_lv,"POST");
		track_action_lv.onLoad = function (success:Boolean) {
			tracker_reference.broadcastReport(success,"trackAction()",currentPage.Addr);
			delete this;
		}
		if (log) {
			broadcastLog("trackAction()",trackerActionURL,track_action_lv);
		}

		}
/**
* TRACK SALE
* @param	currentPage: A String describing the path of a user on a site. Can be an Object {path: [path], title: [title of a page]}
* FUNCTIONAL:
* @param	productsArray: An array of products. An array must consist of Objects: {name: [name of a product], cnt: [count], value: [value])
* @param	orderID
* @param	orderInfo
* @param	orderCost
* @param    saleTags: array of sale tags.
*/
	public function trackSale	(
								lCurrentPage:Object,
								productsArray:Array,
								orderID:String,
								orderInfo:String,
								orderCost:String,
								saleTags:Array
								):Void
		{
		var tracker_reference:Tracker = this;
		var currentPage:Object = parseCurrentPageProperties(lCurrentPage);

		if (currentPage.Addr == undefined || currentPage.Addr == "") {
			broadcastError("ERROR [trackSale()]. CurrentPage Object was not set properly.");
			return;
		}
		var track_sale_lv:LoadVars = new LoadVars();
		// address and title
		track_sale_lv.cur = standartize(currentPage.Addr);
		if (currentPage.Title != null && currentPage.Title!=undefined) track_sale_lv.title = currentPage.Title;
		// products Array
		if (productsArray.length >0) {
			for (var pa=0; pa<productsArray.length; pa++) {
				track_sale_lv["itm["+pa+"]"] = "{{"+productsArray[pa].name+"}}{{"+productsArray[pa].value + "}}{{"+productsArray[pa].cnt + "}}";
			}
		}

		// orderID/info/cost:
		if (orderID.length>0) track_sale_lv.oid = orderID;
		if (orderInfo.length>0) track_sale_lv.oinfo = orderInfo;
		if (orderCost.length>0) track_sale_lv.cs = orderCost;

		if (saleTags.length > 0) track_sale_lv.saleTags = saleTags.join(";");

		addParameters(track_sale_lv);
		track_sale_lv.sendAndLoad(trackerSaleURL,track_sale_lv,"POST");
		track_sale_lv.onLoad = function (success:Boolean) {
			tracker_reference.broadcastReport(success,"trackSales()",currentPage.Addr);
			delete this;
		}
		if (log) {
			broadcastLog("trackSales()",trackerSaleURL,track_sale_lv);
		}

	}

	public function addListener(listener:Object):Void {
		for (var ls=0; ls<_listeners.length; ls++) {
			if (listener == _listeners[ls]) return;
		}
		_listeners.push(listener);
	}
	public function removeListener(listener:Object):Void {
		for (var ls=0; ls<_listeners.length; ls++) {
			if (listener == _listeners[ls]) {
				_listeners.splice(ls,1);
				break;
			}
		}
	}


	// PRIVATES
	private function parseCurrentPageProperties (cp:Object):Object {
		if (typeof(cp) == "string") {
			return {Addr:cp, Title: null};
		} else {
			return {Addr:cp.Addr, Title: cp.Title};
		}
	}
	private function addParameters(lv:LoadVars):Void {
		// adding user parameters
		
		if (siteTextID != undefined && siteTextID != "") {
			lv.stext = siteTextID;
		} else {
			lv.st = siteID.toString();
		}
		for (var props in parameters) {
			if (parameters[props] != null) {
				if (parameters[props] != backup_parameters[props]) {
					lv[props] = parameters[props];
					backup_parameters[props] = parameters[props];
				}
			}
		}

		// adding environment parameters
		flash_parameters.wr = System.capabilities.screenResolutionX + "x" + System.capabilities.screenResolutionY;
		for (var props in flash_parameters) {
			lv[props] = flash_parameters[props];
		}
		lv.charset = _charset;
	}

	private function broadcastError(msg:String):Void {
		for (var ls=0; ls<_listeners.length; ls++) {
			_listeners[ls].onError(msg);
		}

	}
	private function broadcastReport(success:Boolean,type:String, page:String):Void {
		for (var ls=0; ls<_listeners.length; ls++) {
			_listeners[ls].onReport(success, type, page);
		}
	}
	private function broadcastLog (type:String, theURL:String, data:LoadVars):Void {
		for (var ls=0; ls<_listeners.length; ls++) {
				_listeners[ls].onTrack(type, theURL, data);
		}
	}
	private function standartize(str:String):String {
		if (str.substr(0,4)=="www.") str = "http://"+str;
		if (str.substr(0,4)!="http") {
			var string:String = trackerSiteHost;
			
			if (str.substr(0,1) == "/") str = str.substr(1);
			
			str = string + (string.substr(-1)=="/"?(str):("/"+str));
		}
		return str;
	}

	private function checkForHTTP(str:String):String {
		if (str && str.substr(0,7)!="http://") str = "http://"+str;
		return str;
	}
	// GETTERS/SETTERS
	public function set visitorName(val:String) {
		parameters.vname = val;
	}
	public function get visitorName():String {
		return parameters.vname;
	}
	public function set tags(tags:Array) {
		parameters.vtag = tags.join(";");
	}
	public function get tags():Array {
		return parameters.vtag.split(";");
	}
	public function set overwriteTags(val:Boolean) {
		parameters.overwriteTags = val;
	}
	public function get overwriteTags():Boolean {
		return parameters.overwriteTags
	}
	public function set presetIP(val:String) {
		parameters.presetIP = val;
	}
	public function get presetIP():String {
		return parameters.presetIP;
	}
	public function set presetID(val:String) {
		parameters.presetID = val;
	}
	public function get presetID ():String {
		return parameters.presetID;
	}
	public function set charset (val:String) {
		_charset = val;
	}
	public function get charset():String {
		return _charset;
	}
}