<?php

/**
 * @author johann kuindji
 * @author of prototype: Mikolaj Jedrzejak <mikolajj@op.pl>
 *
 * (c) Stuffed Guys
 *
 * example:
 * $Convert= new nsConvertCharset();
 * $Str=$Convert->Convert($Str, "Windows-1251", "UTF-8");
 **/


class nsConvertCharset {

    var $PathToTables;
    var $LastErrorNum;
    var $ReturnErrorFunc;
    var $RecognizedEncoding; //This value keeps information if string contains multibyte chars.
    var $Entities; // This value keeps information if output should be with numeric entities.
    var $ByteRange;
    var $ConvertTable;
    var $Unicode; // 0 - none, 1 - mixed, 2 - full
    var $UnicodeArr;


    /**
     * nsConvertCharset::nsConvertCharset()
     * @param string Path to folder with conversion tables
     * @param string A name of error handler function($ErrorNum, $ErrorValue)
     **/
    function  nsConvertCharset($PathToTables="", $ReturnErrorFunc="")
    {
        $this->LastErrorNum=false;
        if ($PathToTables) $this->PathToTables=$PathToTables;
        $this->ReturnErrorFunc=$ReturnErrorFunc;
        if (!$this->PathToTables) {
            $this->PathToTables = dirname(ereg_replace("\\\\","/",__FILE__))."/tables/";
        }

        $this->ByteRange['utf-8'][0]="191"; // high byte start
        $this->ByteRange['utf-8'][1]="127"; // low byte start
        $this->ByteRange['utf-8'][2]="255"; // high byte end
        $this->ByteRange['utf-8'][3]="191"; // low byte end
        $this->ByteRange['utf-8'][4]="4"; // per char byte limit

        $this->ConvertTable=array();
        $this->Unicode=0;
        $this->UnicodeArr=array();

    }


    /**
     * nsConvertCharset::ReturnError()
     * @param integer Error identifier
     * @param string Error parameter
     **/
    function ReturnError($Number, $Value = false)
    {
        $this->LastErrorNum=$Number;
        $this->LastErrorValue=$Value;
        if (!$this->ReturnErrorFunc) return false;
        $Func=$this->ReturnErrorFunc;
        eval("$Func(\$this->LastErrorNum, \$this->LastErrorValue);");
        return false;
    }

    /**
     * nsConvertCharset::Convert()
     * @param string Text to convert
     * @param string Source charset
     * @param string Destination charset
     **/
    function Convert($Str=false, $FromCharset=false, $ToCharset=false)
    {
        if (!$Str) return false;
        if (!$ToCharset) return false;
        $NewStr="";
        $FromCharset=strtolower($FromCharset);
        $ToCharset=strtolower($ToCharset);
        if (isset($this->Synonyms[$FromCharset])) $FromCharset=$this->Synonyms[$FromCharset];
        if (isset($this->Synonyms[$ToCharset])) $ToCharset=$this->Synonyms[$ToCharset];

        //    %uXXX
        if (ereg('%u([[:alnum:]]{4})', $Str)) {
            $this->Unicode=1; // mixed unicode
            $Arr=array();
            preg_match_all("/%u(\w{4})/e", $Str, $Arr);
            $Arr=$Arr[1];
            if ($Str=="%u".implode("%u",$Arr)) {
                $this->UnicodeArr=$Arr;
                $this->Unicode=2; // full unicode
                if (!$FromCharset) $FromCharset=$ToCharset;
            }
            $NewStr=$this->TryToConvert($Str, $FromCharset, $ToCharset);
            if ($NewStr) return $NewStr;
        }

        if ($this->IsUTF8($Str)) { //   utf-8
            $FromCharset = 'utf-8';
        }
        elseif (!$FromCharset && function_exists("mb_detect_encoding")) {
            //    "",   .
            //     ,    
            $FromCharset=strtolower(@mb_detect_encoding($Str));
            if (isset($this->Synonyms[$FromCharset])) $FromCharset=$this->Synonyms[$FromCharset];
        }
        /*elseif ($FromCharset && function_exists("mb_detect_encoding")) {
            // ,    utf,   "" ,  ,    
            //  , utf   
            //$C = strtolower(mb_detect_encoding($Str, array('Windows-1251'), true));
            $C = strtolower(mb_detect_encoding($Str, array_diff(array($ToCharset, $FromCharset), array('utf-8'))));
            if ($C) $FromCharset = $C;
            else {
                $FromCharset = strtolower(@mb_detect_encoding($Str));
            }
            //   ,    ,     
            if (isset($this->Synonyms[$FromCharset])) $FromCharset=$this->Synonyms[$FromCharset];
        }*/

        if ($FromCharset == $ToCharset)  return $Str;

        if ($FromCharset) {
            $NewStr=$this->TryToConvert($Str, $FromCharset, $ToCharset);
            if ($NewStr) return $NewStr;
        }

        return $Str;
    }


    /**
     * nsConvertCharset::TryToConvert()
     * @param string Text to convert
     * @param string Source charset
     * @param string Destination charset
     **/
    function TryToConvert($Str, $FromCharset, $ToCharset)
    {
        $NewStr=false;
        if (function_exists('mb_convert_encoding')&&!$this->Unicode) $NewStr = @mb_convert_encoding($Str, $ToCharset, $FromCharset);
        if (!$NewStr) $NewStr=$this->DoConvert($Str, $FromCharset, $ToCharset);
        return $NewStr;
    }


	/**
	 * nsConvertCharset::HexToUtf()
	 *
	 * This simple function gets unicode  char up to 4 bytes and return it as a regular char.
	 * It is very similar to  UnicodeEntity function (link below). There is one difference
	 * in returned format. This time it's a regular char(s), in most cases it will be one or two chars.
	 *
	 * @param string $UtfCharInHex Hexadecimal value of a unicode char.
	 * @return string Encoded hexadecimal value as a regular char.
	 * @see UnicodeEntity()
	 **/
	function HexToUtf ($UtfCharInHex)
	{
		$OutputChar = "";
		$UtfCharInDec = hexdec($UtfCharInHex);
		if($UtfCharInDec<128) $OutputChar .= chr($UtfCharInDec);
        else if($UtfCharInDec<2048)$OutputChar .= chr(($UtfCharInDec>>6)+192).chr(($UtfCharInDec&63)+128);
        else if($UtfCharInDec<65536)$OutputChar .= chr(($UtfCharInDec>>12)+224).chr((($UtfCharInDec>>6)&63)+128).chr(($UtfCharInDec&63)+128);
        else if($UtfCharInDec<2097152)$OutputChar .= chr($UtfCharInDec>>18+240).chr((($UtfCharInDec>>12)&63)+128).chr(($UtfCharInDec>>6)&63+128). chr($UtfCharInDec&63+128);
	    return $OutputChar;
	}

    /**
     * nsConvertCharset::IsUTF8()
     *
     * Tries to identify is this string in utf-8 or not
     *
     * @param string
     * @return boolean
     **/
    function IsUTF8($Str=false) {
        if (!$Str) return false;
       // From http://w3.org/International/questions/qa-forms-utf-8.html
       return preg_match('%^(?:
             [\x09\x0A\x0D\x20-\x7E]            # ASCII
           | [\xC2-\xDF][\x80-\xBF]            # non-overlong 2-byte
           |  \xE0[\xA0-\xBF][\x80-\xBF]        # excluding overlongs
           | [\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}  # straight 3-byte
           |  \xED[\x80-\x9F][\x80-\xBF]        # excluding surrogates
           |  \xF0[\x90-\xBF][\x80-\xBF]{2}    # planes 1-3
           | [\xF1-\xF3][\x80-\xBF]{3}          # planes 4-15
           |  \xF4[\x80-\x8F][\x80-\xBF]{2}    # plane 16
       )*$%xs', $Str);
    }



    /**
     * nsConvertCharset::UnicodeToUtf()
     *
     * Converts %uXXXX text to UTF-8
     *
     * @param string
     * @return boolean
     **/
    function UnicodeToUtf($Str=false)
    {
        if (!$Str) return false;
        $Arr=array();
        preg_match_all("/%u(\w{4})/e", $Str, $Arr);
        $Arr=$Arr[1];
        $dest = '';
        foreach ($Arr as $src) {
            $src=hexdec($src);
            if($src < 0) return $Str;
            else if ( $src <= 0x007f) $dest .= chr($src);
            else if ($src <= 0x07ff) {
                $dest .= chr(0xc0 | ($src >> 6));
                $dest .= chr(0x80 | ($src & 0x003f));
            }
            else if($src == 0xFEFF) {
                // nop -- zap the BOM
            }
            else if ($src >= 0xD800 && $src <= 0xDFFF) {
                // found a surrogate
                return $Str;
            }
            else if ($src <= 0xffff) {
                $dest .= chr(0xe0 | ($src >> 12));
                $dest .= chr(0x80 | (($src >> 6) & 0x003f));
                $dest .= chr(0x80 | ($src & 0x003f));
            }
            else if ($src <= 0x10ffff) {
                $dest .= chr(0xf0 | ($src >> 18));
                $dest .= chr(0x80 | (($src >> 12) & 0x3f));
                $dest .= chr(0x80 | (($src >> 6) & 0x3f));
                $dest .= chr(0x80 | ($src & 0x3f));
            }
            else return $Str;
        }
        return $dest;
    }

    # header format (21 bytes total):
    # 4b = STCB (format signature)
    # 4b = High Byte Start range
    # 4b = Low Byte Start range
    # 4b = High Byte End range
    # 4b = Low Byte End range
    # 1b = Maximum Length of one character

    # chars format:
    # 1b - char length
    # 2b - number of consecutive chars with this length
    # "char length" b - charset character
    # 2b - corresponding unicode
    # .. then repeat, first 3 bytes are skipped if the length of the current
    #    character is the same as the length of the previous one

    function MakeConvertTableBin ($Encoding=false) {
        if (!$Encoding) return false;

    	$Filename = $this->PathToTables.$Encoding.'.bin';
    	$FileBin = @fopen($Filename, 'rb');
    	if (!$FileBin) return $this->ReturnError( 0, $this->PathToTables.$Encoding.'.bin'); //Print an error message
    	$contents = fread($FileBin, filesize($Filename));
    	fclose($FileBin);

    	// Checking file signature
    	if (substr($contents, 0, 4) != 'STCB') return false;

    	$this->ByteRange[$Encoding] = array();

    	// Reading the header
    	for ($set = 0; $set <= 3; $set++) {
    		for ($pos = 4+$set*4; $pos <= 7+$set*4; $pos++) {
    			$byte = ord(substr($contents, $pos, 1));
    			if ($byte == 0) continue;
    			if (@$this->ByteRange[$Encoding][$set]) $this->ByteRange[$Encoding][$set] .= '|';
    	       	else $this->ByteRange[$Encoding][$set] = $byte;
    		}
    	}

        // Chars maximum length
    	$char_max = ord(substr($contents, 20, 1));
    	$this->ByteRange[$Encoding][4] = $char_max;

    	// Setting position at the first byte after the header
    	$pos = 21;
    	$char_seq = 0;
    	$char_length = 0;
    	while ($pos < strlen($contents)-1) {
    		// Only read char length information if we don't have it already from the
    		// previous char
    		if ($char_seq == 0) {
    			$char_length = ord(substr($contents, $pos, 1));
    			$char_seq = unpack("S", substr($contents, $pos+1, 2));
    			$pos += 3;
    		}

    		$org_char = strtoupper(implode(unpack('H'.($char_length*2), substr($contents, $pos, $char_length))));
    		$pos += $char_length;

    		if ($char_max > $char_length)		$org_char=sprintf("%0".($char_max*2)."s", $org_char);

    		// Just change the max number in byte range, because the actual length
    		// of every char is stored separately for every char in the bin file, no
    		// need to do a new reading of the same bin file again
    		if ($char_max < $char_length) $this->ByteRange[$Encoding][4] = $char_max;

    		$uni_char = strtoupper(implode(unpack('H4', substr($contents, $pos, 2))));
    		$pos += 2;

    		// Decreasing the number of consecutive chars with the same length, since
    		// we've just processed one char
    		$char_seq--;

    		$this->ConvertTable[$Encoding][$uni_char] = $org_char;
    	}
	}

	/**
	 * nsConvertCharset::MakeConvertTable()
	 *
	 * This function creates table with two SBCS (Single Byte Character Set). Every conversion
	 * is through this table.
	 *
	 * - The file with encoding tables have to be save in "Format A" of unicode.org charset table format! This is usualy writen in a header of every charset file.
	 * - BOTH charsets MUST be SBCS
	 * - The files with encoding tables have to be complet (Non of chars can be missing, unles you are sure you are not going to use it)
	 *
	 * "Format A" encoding file, if you have to build it by yourself should aplly these rules:
	 * - you can comment everything with #
	 * - first column contains 1 byte chars in hex starting from 0x..
	 * - second column contains unicode equivalent in hex starting from 0x....
	 * - then every next column is optional, but in "Format A" it should contain unicode char name or/and your own comment
	 * - the columns can be splited by "spaces", "tabs", "," or any combination of these
	 * - below is an example
	 *
	 * <code>
	 * #
	 * #	The entries are in ANSI X3.4 order.
	 * #
	 * 0x00	0x0000	#	NULL end extra comment, if needed
	 * 0x01	0x0001	#	START OF HEADING
	 * # Oh, one more thing, you can make comments inside of a rows if you like.
	 * 0x02	0x0002	#	START OF TEXT
	 * 0x03	0x0003	#	END OF TEXT
	 * next line, and so on...
	 * </code>
	 *
	 * You can get full tables with encodings from http://www.unicode.org
	 *
	 * @param string $FirstEncoding Name of first encoding and first encoding filename (thay have to be the same) or array
	 * @return void
	 **/

	function MakeConvertTable ($FirstEncoding) {
		$ConvertTable = array();
		$FilesArr=array();

		if (is_array($FirstEncoding)) $FilesArr=$FirstEncoding;
		else for($i = 0; $i < func_num_args(); $i++) $FilesArr[] = func_get_arg($i);

		foreach($FilesArr as $i=>$FileName)	{

			if (isset($this->ConvertTable[$FileName])) continue;

            // if binary table
            if (is_file($this->PathToTables.$FileName.'.bin')) $this->MakeConvertTableBin($FileName);

            if (!isset($this->ConvertTable[$FileName]) && !is_file($this->PathToTables.$FileName)) {
				return $this->ReturnError( 0, $this->PathToTables.$FileName); //Print an error message
			}
            if (isset($this->ConvertTable[$FileName])) continue;

			$FileWithEncTabe = fopen($this->PathToTables.$FileName, "r");
			if (!$FileWithEncTabe) return false;

			while(!feof($FileWithEncTabe)) {

                //We asume that line is not longer than 1024 which is the default value for fgets function
				$OneLine=trim(fgets($FileWithEncTabe, 1024));
                if (!$OneLine) continue;

				if (substr($OneLine, 0, 2)=="--") {
					if (!@$this->ByteRange[$FileName]) $this->ByteRange[$FileName]=explode(",",substr(str_replace(" ","",$OneLine),2));
					continue;
                }

				if (!@$this->ByteRange[$FileName]) $this->ByteRange[$FileName][4]=1;

				//We don't need all comment lines. I check only for "#" sign, because this is a way of making comments
				//by unicode.org in thair encoding files and that's where the files are from :-)
				if (substr($OneLine, 0, 1) == "#") continue;

				//Sometimes inside the charset file the hex walues are separated by "space" and sometimes by "tab",
				//the below preg_split can also be used to split files where separator is a ",", "\r", "\n" and "\f"
				$HexValue = preg_split ("/[\s,]+/", $OneLine, 3);  //We need only first 2 values

				//Sometimes char is UNDEFINED, or missing so we can't use it for convertion
				if (@substr($HexValue[1], 0, 1) == "#" || !isset($HexValue[1])) continue;

				$ArrayKey = strtoupper(str_replace(strtolower("0x"), "", $HexValue[1]));
				$ArrayValue = strtoupper(str_replace(strtolower("0x"), "", $HexValue[0]));

				if ($this->ByteRange[$FileName][4] && $this->ByteRange[$FileName][4]>strlen($ArrayValue)/2) {
					$ArrayValue=sprintf("%0".($this->ByteRange[$FileName][4]*2)."s", $ArrayValue);
				}

				if ($this->ByteRange[$FileName][4] < strlen($ArrayValue)/2) {
					unset($this->ConvertTable[$FileName]);
					$this->ByteRange[$FileName][4]=strlen($ArrayValue)/2;
					$this->MakeConvertTable($FileName);
					break;
				}

				$this->ConvertTable[$FileName][$ArrayKey] = $ArrayValue;

			} //while(!feof($FirstFileWi...
		} //for($i = 0; $i < func_...
	}



    function Char($Hex)
    {
        $Byte=ceil(strlen($Hex)/2);
        $Char="";
        for ($i=0;$i<$Byte;$i++) $Char.=(substr($Hex,$i*2,2)=="00")?"": chr(hexdec(substr($Hex,$i*2,2)));
        return $Char;
    }

    function UtfCharToUnicode($Bytes,$Hex,$Position)
    {
        $BytesArr=array();
        for ($i=0;$i<$Bytes;$i++) {
            $Char = substr($Hex, $Position*2,2);
            $AsciiChar = hexdec ($Char);
            if ($i==0) $BytesArr[]=($AsciiChar & 31);
            else $BytesArr[]=($AsciiChar & 63);
            $Position++;
        }
        $Byte=0;
        for($i=0;$i<count($BytesArr);$i++) $Byte=($Byte*64)+$BytesArr[$i];
        return dechex($Byte);
    }

    function UtfToUnicode($Hex)
    {
        $OutString  = "";
        $StringLenght = strlen($Hex)/2;
        for ($CharPosition = 0; $CharPosition < $StringLenght; $CharPosition++) {
            $Char = substr($Hex, $CharPosition*2,2);
            $AsciiChar=hexdec($Char);
            //  1 7 0bbbbbbb (127)
            if ($AsciiChar < 128) $OutString .= $Char;
            //2 11 110bbbbb 10bbbbbb (2047)
            else if ($AsciiChar >> 5 == 6)  $OutString .= $this->UtfCharToUnicode(2,$Hex,$CharPosition);
            //3 16 1110bbbb 10bbbbbb 10bbbbbb
            else if ($AsciiChar >> 4  == 14)  $OutString .= $this->UtfCharToUnicode(3,$Hex,$CharPosition);
             //4 21 11110bbb 10bbbbbb 10bbbbbb 10bbbbbb
             else if ($AsciiChar >> 3 == 30) $OutString .= $this->UtfCharToUnicode(4,$Hex,$CharPosition);
        }
        return $OutString;
    }

    function GetCharsetByteRange($FromCharset,&$HighByteStart,&$LowByteStart,&$HighByteEnd,&$LowByteEnd,&$ByteLimit)
    {
        $HighByteStart=@$this->ByteRange[$FromCharset][0] or 0;
        $LowByteStart=@$this->ByteRange[$FromCharset][1] or 0;
        $HighByteEnd=@$this->ByteRange[$FromCharset][2] or 0;
        $LowByteEnd=@$this->ByteRange[$FromCharset][3] or 0;
        $ByteLimit=@$this->ByteRange[$FromCharset][4] or 2;
        $HighByteStart=explode("|",$HighByteStart);
        $HighByteEnd=explode("|",$HighByteEnd);
        $LowByteStart=explode("|",$LowByteStart);
        $LowByteEnd=explode("|",$LowByteEnd);
    }

    function InByteRange($Int, $Start, $End)
    {
        foreach ($Start as $i=>$v) if ($Int > $Start[$i] && $Int <= $End[$i]) return true;
        return false;
    }

    function StringToHex($Str="", $FromCharset="")
    {
        $HexArr=array();
        $MaxByte=1;
        $HighByteStart=$LowByteStart=$HighByteEnd=$LowByteEnd=$ByteLimit=false;
        $this->GetCharsetByteRange($FromCharset,$HighByteStart,$LowByteStart,$HighByteEnd,$LowByteEnd,$ByteLimit);

        for ($i = 0; $i < strlen($Str); $i++) {
            //      unicode %u....
            if ($this->Unicode>0 && $Str[$i]=="%" && preg_match("/%u(\w{4})/e",substr($Str,$i,6))) {
                $UnicodeKey=str_replace("%u","",substr($Str,$i,6));
                if ($FromCharset!='utf-8') {
                    if (isset($this->ConvertTable[$FromCharset][$UnicodeKey])) $HexArr[]=$this->ConvertTable[$FromCharset][$UnicodeKey];
                    else $HexArr[]="&#".hexdec($UnicodeKey);
                }
                if ($FromCharset=='utf-8') $HexArr[]=$UnicodeKey;
                $i+=5;
                continue;
            }
            if ($FromCharset!='utf-8' && !isset($this->ByteRange[$FromCharset][0])) {
                $HexArr[]=strtoupper(dechex(ord($Str[$i])));
                continue;
            }
            $Char=ord($Str[$i]);
            $HexChar=dechex($Char);
            $CurByte=1;
            if ($this->InByteRange($Char,$HighByteStart,$HighByteEnd)) {//  
                for ($j=$i+1;$j<strlen($Str);$j++) {
                    if ($CurByte==$ByteLimit) break;
                    $_Char=ord($Str[$j]);
                    if ($this->InByteRange($_Char,$LowByteStart,$LowByteEnd)) {//  
                        $HexChar.=dechex($_Char);
                        $i++;
                        $CurByte++;
                    } else break;
                }
            }
            $MaxByte=max($MaxByte,$CurByte);
            if ($FromCharset=='utf-8') $HexChar=$this->UtfToUnicode($HexChar);
            $HexArr[]=$HexChar;
        }
        if ($FromCharset=='utf-8') $MaxByte=2; // utf is converted to unicode (2byte) by default
        for ($i=0;$i<Count($HexArr);$i++) $HexArr[$i]=strtoupper(sprintf("%0".($MaxByte*2)."s", $HexArr[$i]));

        return $HexArr;
    }


	/**
	 * nsConvertCharset::DoConvert()
	 *
	 * This is a basic function you are using. I hope that you can figure out this function syntax :-)
	 *
	 * @param string $StringToChange The string you want to change :)
	 * @param string $FromCharset Name of $StringToChange encoding, you have to know it.
	 * @param string $ToCharset Name of a charset you want to get for $StringToChange.
	 * @param boolean $TurnOnEntities Set to true or 1 if you want to use numeric entities insted of regular chars.
	 * @return string Converted string in brand new encoding :)
	 * @version 1.0 2004-07-27 01:09
	 **/
	function DoConvert ($StringToChange="", $FromCharset="", $ToCharset="")
	{
        //Check are there all variables
		if ($StringToChange == "")	return $this->ReturnError(3, "\$StringToChange");
		if ($FromCharset == "" && $this->Unicode!=2) return $this->ReturnError(3, "\$FromCharset");
		if ($ToCharset == "") return $this->ReturnError(3, "\$ToCharset");
		$NewString = "";

		$FromCharset = strtolower($FromCharset);
		$ToCharset   = strtolower($ToCharset);

		if ($FromCharset == $ToCharset && !$this->Unicode) return $StringToChange;

		// This is the first case. We are convertinf from non-utf8
		if ($FromCharset != "utf-8")
		{
				// Now build table with both charsets for encoding change.
                if ($FromCharset == 'utf-8' && $ToCharset == 'unicode') {}
                else {
				    if ($ToCharset != "utf-8" && $ToCharset != 'unicode') $this->MakeConvertTable ($FromCharset, $ToCharset);
				    else $this->MakeConvertTable($FromCharset);
                    if (empty($this->ConvertTable)) return $this->ReturnError(0, "$FromCharset - $ToCharset");
                }

                $HexArr=$this->StringToHex($StringToChange, $FromCharset);

				//For each char in a string...
				foreach ($HexArr as $HexChar) {

                    // entity
                    if (preg_match("/&#(\w{4})/e",$HexChar)) {
                        $NewString.=$HexChar;
                        continue;
                    }

                    if (!in_array($HexChar, @$this->ConvertTable[$FromCharset])) {
                        $NewString.=$this->Char($HexChar);
                        continue;
                    }

					if ($ToCharset != "utf-8") {
						$UnicodeHexChar = array_search($HexChar, $this->ConvertTable[$FromCharset]);
						$UnicodeHexChars = explode("+",$UnicodeHexChar);
						for($UnicodeHexCharElement = 0; $UnicodeHexCharElement < count($UnicodeHexChars); $UnicodeHexCharElement++)
						{
                            if (array_key_exists($UnicodeHexChars[$UnicodeHexCharElement], $this->ConvertTable[$ToCharset]))	{

								$NewString .= $this->Char($this->ConvertTable[$ToCharset][$UnicodeHexChars[$UnicodeHexCharElement]]);
                            }
                            else $NewString.="&#".hexdec($UnicodeHexChars[$UnicodeHexCharElement]);
                            //else $NewString.=$this->Char($HexChar);

						} //for($UnicodeH...
					}
					else
					{
						$UnicodeHexChar = array_search($HexChar, $this->ConvertTable[$FromCharset]);
							/**
    					     * Sometimes there are two or more utf-8 chars per one regular char.
    							 * Extream, example is polish old Mazovia encoding, where one char contains
    							 * two lettes 007a (z) and 0142 (l slash), we need to figure out how to
    							 * solve this problem.
    							 * The letters are merge with "plus" sign, there can be more than two chars.
    							 * In Mazowia we have 007A+0142, but sometimes it can look like this
    							 * 0x007A+0x0142+0x2034 (that string means nothing, it just shows the possibility...)
    					     **/
						$UnicodeHexChars = explode("+",$UnicodeHexChar);
						for($UnicodeHexCharElement = 0; $UnicodeHexCharElement < count($UnicodeHexChars); $UnicodeHexCharElement++)
						{
							$NewString .= $this->HexToUtf($UnicodeHexChars[$UnicodeHexCharElement]);
                        }
					}
				}
		}
		 // This is second case. We are encoding from utf-8.
		else if($FromCharset == "utf-8" && $ToCharset != 'unicode')
		{
            $this->MakeConvertTable($ToCharset);
            $HexArr=$this->StringToHex($StringToChange, $FromCharset);
            if (empty($this->ConvertTable)) return $this->ReturnError(0, $ToCharset);
            $NewString="";

            foreach ($HexArr as $HexChar) {
                // entity
                if (preg_match("/&#(\w{4})/e",$HexChar)) {
                    $NewString.=$HexChar;
                    continue;
                }
                if (hexdec($HexChar)<128) $NewString.=$this->Char($HexChar);
                else $NewString.= $this->Char( (array_key_exists($HexChar, $this->ConvertTable[$ToCharset]))?
                                                $this->ConvertTable[$ToCharset][$HexChar]:
                                                $HexChar );
            }
		}
        else if ($FromCharset == 'utf-8' && $ToCharset == 'unicode') {
            $HexArr=$this->StringToHex($StringToChange, $FromCharset);
            $NewString="";
            foreach ($HexArr as $HexChar) {
                //$NewString .= dechex(hexdec($HexChar));
                $NewString .= "\x".$HexChar;
            }
        }
	    return $NewString;
	}



    var $Synonyms=array(
        'us' => 'ascii',
        'us-ascii'=> 'ascii',
        'cp819' => 'iso-8859-1',
        'ibm819' => 'iso-8859-1',
        'iso-ir-100' => 'iso-8859-1',
        'iso-ir-109' => 'iso-8859-2',
        'iso-ir-148' => 'iso-8859-9',
        'iso-ir-199' => 'iso-8859-14',
        'iso-ir-203' => 'iso-8859-15',
        'csisolatin1' => 'iso-8859-1',
        'csisolatin2' => 'iso-8859-2',
        'csisolatin3' => 'iso-8859-3',
        'csisolatin5' => 'iso-8859-9',
        'csisolatin8' => 'iso-8859-14',
        'csisolatin9' => 'iso-8859-15',
        'csisolatingreek' => 'iso-8859-7',
        'iso-celtic' => 'iso-8859-14',
        'latin1' => 'iso-8859-1',
        'latin2' => 'iso-8859-2',
        'latin3' => 'iso-8859-3',
        'latin5' => 'iso-8859-9',
        'latin6' => 'iso-8859-10',
        'latin8' => 'iso-8859-14',
        'latin9' => 'iso-8859-15',
        'l1' => 'iso-8859-1',
        'l2' => 'iso-8859-2',
        'l3' => 'iso-8859-3',
        'l5' => 'iso-8859-9',
        'l6' => 'iso-8859-10',
        'l8' => 'iso-8859-14',
        'l9' => 'iso-8859-15',
        'cyrillic' => 'iso-8859-5',
        'arabic' => 'iso-8859-6',
        'tis-620' => 'iso-8859-11',
        'win874' => 'windows-874',
        'win1250' => 'windows-1250',
        'win1251' => 'windows-1251',
        'win1252' => 'windows-1252',
        'win1253' => 'windows-1253',
        'win1254' => 'windows-1254',
        'win1255' => 'windows-1255',
        'win1256' => 'windows-1256',
        'win1257' => 'windows-1257',
        'win1258' => 'windows-1258',
        'cp1250' => 'windows-1250',
        'cp1251' => 'windows-1251',
        'cp1252' => 'windows-1252',
        'ms-ee' => 'windows-1250',
        'ms-ansi' => 'windows-1252',
        'ms-greek' => 'windows-1253',
        'ms-turk' => 'windows-1254',
        'winbaltrim' => 'windows-1257',
        'koi-8ru' => 'koi-8r',
        'koi8r' => 'koi-8r',
        'cp878' => 'koi-8r',
        'mac' => 'macroman',
        'macintosh' => 'macroman',
        'euc-cn' => 'gb2312',
        'x-euc-cn' => 'gb2312',
        'euccn' => 'gb2312',
        'cp936' => 'gb2312',
        'big-5' => 'big5',
        'cp950' => 'big5',
        'eucjp' => 'euc-jp',
        'sjis' => 'shift_jis',
        'shift-jis' => 'shift_jis',
        'cp932' => 'shift_jis',
        'cp949' => 'euc-kr',
        'utf7' => 'utf-7',
        'utf8' => 'utf-8',
        'utf16' => 'utf-16',
        'utf32' => 'utf-32',
        'ucs2' => 'ucs-2',
        'ucs4' => 'ucs-4',
    );


} //class ends here
?>